@@ -42,8 +42,9 @@ Rcpp::List new_KrigingFit(arma::vec y,
     if (params.containsElementNamed("sigma2")) {
       _parameters.push_back(params["sigma2"], "sigma2");
       _parameters.push_back(true, "has_sigma2");
-      _parameters.push_back(!(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]),
-                            "is_sigma2_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]) && optim != "none",
+          "is_sigma2_estim");
     } else {
       //_parameters.push_back(Rcpp::runif(1), "sigma2"); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(true, "has_sigma2");
@@ -52,10 +53,15 @@ Rcpp::List new_KrigingFit(arma::vec y,
       _parameters.push_back(true, "is_sigma2_estim");
     }
     if (params.containsElementNamed("theta")) {
-      _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["theta"]), "theta");
+      Rcpp::NumericVector theta = Rcpp::as<Rcpp::NumericVector>(params["theta"]);
+      if (!theta.hasAttribute("dim")) {
+        theta.attr("dim") = Rcpp::Dimension(1, theta.length());
+      }
+      _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(theta), "theta");
       _parameters.push_back(true, "has_theta");
-      _parameters.push_back(!(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]),
-                            "is_theta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]) && optim != "none",
+          "is_theta_estim");
     } else {
       // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(Rcpp::NumericMatrix(1, X.n_cols,
@@ -67,8 +73,9 @@ Rcpp::List new_KrigingFit(arma::vec y,
     if (params.containsElementNamed("beta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["beta"]), "beta");
       _parameters.push_back(true, "has_beta");
-      _parameters.push_back(!(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]),
-                            "is_beta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]) && optim != "none",
+          "is_beta_estim");
     } else {
       _parameters.push_back(Rcpp::NumericVector(0), "beta");
       _parameters.push_back(false, "has_beta");
@@ -137,8 +144,9 @@ void kriging_fit(Rcpp::List k,
     if (params.containsElementNamed("sigma2")) {
       _parameters.push_back(params["sigma2"], "sigma2");
       _parameters.push_back(true, "has_sigma2");
-      _parameters.push_back(!(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]),
-                            "is_sigma2_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]) && optim != "none",
+          "is_sigma2_estim");
     } else {
       //_parameters.push_back(Rcpp::runif(1), "sigma2"); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(true, "has_sigma2");
@@ -149,8 +157,9 @@ void kriging_fit(Rcpp::List k,
     if (params.containsElementNamed("theta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["theta"]), "theta");
       _parameters.push_back(true, "has_theta");
-      _parameters.push_back(!(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]),
-                            "is_theta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]) && optim != "none",
+          "is_theta_estim");
     } else {
       // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(Rcpp::NumericMatrix(1, X.n_cols,
@@ -162,8 +171,9 @@ void kriging_fit(Rcpp::List k,
     if (params.containsElementNamed("beta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["beta"]), "beta");
       _parameters.push_back(true, "has_beta");
-      _parameters.push_back(!(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]),
-                            "is_beta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]) && optim != "none",
+          "is_beta_estim");
     } else {
       _parameters.push_back(Rcpp::NumericVector(0), "beta");
       _parameters.push_back(false, "has_beta");
@@ -263,23 +273,27 @@ std::string kriging_summary(Rcpp::List k) {
 }
 
 // [[Rcpp::export]]
-Rcpp::List kriging_predict(Rcpp::List k, arma::mat X, bool stdev = true, bool cov = false, bool deriv = false) {
+Rcpp::List kriging_predict(Rcpp::List k,
+                           arma::mat X_n,
+                           bool return_stdev = true,
+                           bool return_cov = false,
+                           bool return_deriv = false) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  auto pred = impl_ptr->predict(X, stdev, cov, deriv);
+  auto pred = impl_ptr->predict(X_n, return_stdev, return_cov, return_deriv);
 
   Rcpp::List ret = Rcpp::List::create(Rcpp::Named("mean") = std::get<0>(pred));
-  if (stdev) {
+  if (return_stdev) {
     ret.push_back(std::get<1>(pred), "stdev");
   }
-  if (cov) {
+  if (return_cov) {
     ret.push_back(std::get<2>(pred), "cov");
   }
-  if (deriv) {
+  if (return_deriv) {
     ret.push_back(std::get<3>(pred), "mean_deriv");
     ret.push_back(std::get<4>(pred), "stdev_deriv");
   }
@@ -288,25 +302,36 @@ Rcpp::List kriging_predict(Rcpp::List k, arma::mat X, bool stdev = true, bool co
 }
 
 // [[Rcpp::export]]
-arma::mat kriging_simulate(Rcpp::List k, int nsim, int seed, arma::mat X) {
+arma::mat kriging_simulate(Rcpp::List k, int nsim, int seed, arma::mat X_n, bool will_update = false) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  return impl_ptr->simulate(nsim, seed, X);
+  return impl_ptr->simulate(nsim, seed, X_n, will_update);
 }
 
 // [[Rcpp::export]]
-void kriging_update(Rcpp::List k, arma::vec y, arma::mat X) {
+arma::mat kriging_update_simulate(Rcpp::List k, arma::vec y_u, arma::mat X_u) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  impl_ptr->update(y, X);
+  return impl_ptr->update_simulate(y_u, X_u);
+}
+
+// [[Rcpp::export]]
+void kriging_update(Rcpp::List k, arma::vec y_u, arma::mat X_u, bool refit = true) {
+  if (!k.inherits("Kriging"))
+    Rcpp::stop("Input must be a Kriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<Kriging> impl_ptr(impl);
+
+  impl_ptr->update(y_u, X_u, refit);
 
   // Rcpp::List obj;
   // obj.attr("object") = impl_ptr;
@@ -315,20 +340,46 @@ void kriging_update(Rcpp::List k, arma::vec y, arma::mat X) {
 }
 
 // [[Rcpp::export]]
-Rcpp::List kriging_logLikelihoodFun(Rcpp::List k, arma::vec theta, bool grad = false, bool hess = false) {
+void kriging_save(Rcpp::List k, std::string filename) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  std::tuple<double, arma::vec, arma::mat> ll = impl_ptr->logLikelihoodFun(theta, grad, hess);
+  return impl_ptr->save(filename);
+}
+
+// [[Rcpp::export]]
+arma::mat kriging_covMat(Rcpp::List k, arma::mat X1, arma::mat X2) {
+  if (!k.inherits("Kriging"))
+    Rcpp::stop("Input must be a Kriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<Kriging> impl_ptr(impl);
+
+  return impl_ptr->covMat(X1, X2);
+}
+
+// [[Rcpp::export]]
+Rcpp::List kriging_logLikelihoodFun(Rcpp::List k,
+                                    arma::vec theta,
+                                    bool return_grad = false,
+                                    bool return_hess = false,
+                                    bool bench = false) {
+  if (!k.inherits("Kriging"))
+    Rcpp::stop("Input must be a Kriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<Kriging> impl_ptr(impl);
+
+  std::tuple<double, arma::vec, arma::mat> ll = impl_ptr->logLikelihoodFun(theta, return_grad, return_hess, bench);
 
   Rcpp::List ret = Rcpp::List::create(Rcpp::Named("logLikelihood") = std::get<0>(ll));
-  if (grad) {
+  if (return_grad) {
     ret.push_back(std::get<1>(ll), "logLikelihoodGrad");
   }
-  if (hess) {
+  if (return_hess) {
     ret.push_back(std::get<2>(ll), "logLikelihoodHess");
   }
 
@@ -347,23 +398,39 @@ double kriging_logLikelihood(Rcpp::List k) {
 }
 
 // [[Rcpp::export]]
-Rcpp::List kriging_leaveOneOutFun(Rcpp::List k, arma::vec theta, bool grad = false) {
+Rcpp::List kriging_leaveOneOutFun(Rcpp::List k, arma::vec theta, bool return_grad = false, bool bench = false) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  std::tuple<double, arma::vec> loo = impl_ptr->leaveOneOutFun(theta, grad);
+  std::tuple<double, arma::vec> loo = impl_ptr->leaveOneOutFun(theta, return_grad, bench);
 
   Rcpp::List ret = Rcpp::List::create(Rcpp::Named("leaveOneOut") = std::get<0>(loo));
-  if (grad) {
+  if (return_grad) {
     ret.push_back(std::get<1>(loo), "leaveOneOutGrad");
   }
 
   return ret;
 }
 
+// [[Rcpp::export]]
+Rcpp::List kriging_leaveOneOutVec(Rcpp::List k, arma::vec theta) {
+  if (!k.inherits("Kriging"))
+    Rcpp::stop("Input must be a Kriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<Kriging> impl_ptr(impl);
+
+  std::tuple<arma::vec, arma::vec> yhat = impl_ptr->leaveOneOutVec(theta);
+
+  Rcpp::List ret
+      = Rcpp::List::create(Rcpp::Named("mean") = std::get<0>(yhat), Rcpp::Named("stdev") = std::get<1>(yhat));
+
+  return ret;
+}
+
 // [[Rcpp::export]]
 double kriging_leaveOneOut(Rcpp::List k) {
   if (!k.inherits("Kriging"))
@@ -376,17 +443,17 @@ double kriging_leaveOneOut(Rcpp::List k) {
 }
 
 // [[Rcpp::export]]
-Rcpp::List kriging_logMargPostFun(Rcpp::List k, arma::vec theta, bool grad = false) {
+Rcpp::List kriging_logMargPostFun(Rcpp::List k, arma::vec theta, bool return_grad = false, bool bench = false) {
   if (!k.inherits("Kriging"))
     Rcpp::stop("Input must be a Kriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<Kriging> impl_ptr(impl);
 
-  std::tuple<double, arma::vec> lmp = impl_ptr->logMargPostFun(theta, grad);
+  std::tuple<double, arma::vec> lmp = impl_ptr->logMargPostFun(theta, return_grad, bench);
 
   Rcpp::List ret = Rcpp::List::create(Rcpp::Named("logMargPost") = std::get<0>(lmp));
-  if (grad) {
+  if (return_grad) {
     ret.push_back(std::get<1>(lmp), "logMargPostGrad");
   }
 
