@@ -1,7 +1,5 @@
-library(testthat)
-
 #library(rlibkriging, lib.loc="bindings/R/Rlibs")
-#library(rlibkriging)
+#library(testthat)
 
 # f <- function(X) apply(X, 1, function(x) prod(sin((x-.5)^2)))
 f <- function(X) apply(X, 1, function(x)
@@ -20,9 +18,10 @@ points(X)
 k <- Kriging(y, X,"gauss",parameters = list(theta=matrix(runif(40),ncol=2)))
 print(k)
 
-rlibkriging::save(k, filename="k.h5")
+unlink("k.json")
+rlibkriging::save(k, filename="k.json")
 
-k2 <- rlibkriging::load(filename="k.h5")
+k2 <- rlibkriging::load(filename="k.json")
 print(k2)
 
 test_that("Save/Load NuggetKriging", expect_true( print(k) == print(k2)))
@@ -30,9 +29,10 @@ test_that("Save/Load NuggetKriging", expect_true( print(k) == print(k2)))
 nuk <- NuggetKriging(y, X,"gauss",parameters = list(theta=matrix(runif(40),ncol=2)))
 print(nuk)
 
-rlibkriging::save(nuk, filename="nuk.h5")
+unlink("nuk.json")
+rlibkriging::save(nuk, filename="nuk.json")
 
-nuk2 <- rlibkriging::load(filename="nuk.h5")
+nuk2 <- rlibkriging::load(filename="nuk.json")
 print(nuk2)
 
 test_that("Save/Load NuggetKriging", expect_true( print(nuk) == print(nuk2)))
@@ -40,9 +40,10 @@ test_that("Save/Load NuggetKriging", expect_true( print(nuk) == print(nuk2)))
 nok <- NoiseKriging(y, rep(0.1^2,nrow(X)), X,"gauss",parameters = list(theta=matrix(runif(40),ncol=2)))
 print(nok)
 
-rlibkriging::save(nok, filename="nok.h5")
+unlink("nok.json")
+rlibkriging::save(nok, filename="nok.json")
 
-nok2 <- rlibkriging::load(filename="nok.h5")
+nok2 <- rlibkriging::load(filename="nok.json")
 print(nok2)
 
 test_that("Save/Load NoiseKriging", expect_true( print(nok) == print(nok2)))
